<?php

namespace App\Http\Controllers\api;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use App\Models\User;
use App\Models\UserMobileProfile;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use App\Http\Requests\StoreUserRequest;
use App\Http\Requests\UpdateUserRequest;
use Laravel\Sanctum\HasApiTokens;

class UsersController extends Controller
{
    /**
     * Display all users
     * 
     * @return \Illuminate\Http\Response
     */
    public function index() 
    {
        $users = User::latest()->paginate(10);

        return view('users.index', compact('users'));
    }

    /**
     * Show form for creating user
     * 
     * @return \Illuminate\Http\Response
     */
    public function create() 
    {
        return view('users.create');
    }

    /**
     * Store a newly created user
     * 
     * @param User $user
     * @param StoreUserRequest $request
     * 
     * @return \Illuminate\Http\Response
     */
    public function store(User $user, StoreUserRequest $request) 
    {
        //For demo purposes only. When creating user or inviting a user
        // you should create a generated random password and email it to the user
        $user->create(array_merge($request, [
            'password' => 'test' 
        ]));

        return redirect()->route('users.index')
            ->withSuccess(__('User created successfully.'));
    }

    /**
     * Show user data
     * 
     * @param User $user
     * 
     * @return \Illuminate\Http\Response
     */
    public function show(User $user) 
    {
        return view('users.show', [
            'user' => $user
        ]);
    }

    /**
     * Edit user data
     * 
     * @param User $user
     * 
     * @return \Illuminate\Http\Response
     */
    public function edit(User $user) 
    {
        return view('users.edit', [
            'user' => $user,
            'userRole' => $user->roles->pluck('name')->toArray(),
            'roles' => Role::latest()->get()
        ]);
    }

    /**
     * Update user data
     * 
     * @param User $user
     * @param UpdateUserRequest $request
     * 
     * @return \Illuminate\Http\Response
     */
    public function update(User $user, UpdateUserRequest $request) 
    {
        $user->update($request->validated());

        $user->syncRoles($request->get('role'));

        return redirect()->route('users.index')
            ->withSuccess(__('User updated successfully.'));
    }

    /**
     * Delete user data
     * 
     * @param User $user
     * 
     * @return \Illuminate\Http\Response
     */
    public function destroy(User $user) 
    {
        $user->delete();

        return redirect()->route('users.index')
            ->withSuccess(__('User deleted successfully.'));
    }
    
    
    /**
     * sign up user $user
     * 
     * @param Request $request
     * 
     * @return \Illuminate\Http\Response
     */
    public function register(Request $request) {

        $userCheck = User::where('email', $request->email)->count();
        if($userCheck>0)
        {
             return response()->json(["success"=>false,"errors"=>['email'=>['email already exists']]]);
        }
        
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required|email',
            'password' => 'required',
        ]);
 
        if ($validator->fails()) {
            return response()->json(["success"=>false,"errors"=>$validator->errors()]);
        }
        
        $user = new User();
        $user->name = $request->name;
        $user->email = $request->email;
        $user->password = bcrypt($request->password);
        $user->user_type = "mobile";
        $user->device_token = $request->device_token;
        if($user->save())
        {
            $prof = new UserMobileProfile;
            $prof->user_id=$user->id;
            $prof->save(); //saving mobile profile
            
            return response()->json(["success"=>true,"msg"=>"User Successfully register!","user"=>$user,"token"=>$user->createToken($user->email)->plainTextToken]);
        }else
        {
            return response()->json(["success"=>false,"errors"=>['user'=>'Error while saving user']]);
        }
        
       
        
    }
    
    /**
     * Login user $user
     * 
     * @param Request $request
     * 
     * @return \Illuminate\Http\Response
     */
    public function login(Request $request) {
    
    //print_r($request->all());
        
        $validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'password' => 'required',
        ]);
 
        if ($validator->fails()) {
            return response()->json(["success"=>false,"msg"=>"The provided credentials do not match our records"]);
        }
        
        $user = User::where('email', $request->email)->first();
 
        if($user && Hash::check($request->password, $user->password)){
            //return $user;
            User::where("id",$user->id)->update(["device_token"=>$request->device_token]);
            return response()->json(["success"=>true,"token"=>$user->createToken($request->email)->plainTextToken,"user"=>$user]);
        }
      
        
          return response()->json(["success"=>false,"msg"=>"The provided credentials do not match our records"]);
        
    }
    
    /**
     * Logout  user $user
     * 
     * @param Request $request
     * 
     * @return \Illuminate\Http\Response
     */
    
    public function logout(Request $request){
        
        $user = $request->user();
        $user->tokens()->delete();
        return response()->json(["success"=>true,"msg"=>"Successfullly logged out!"]);
    }
    
     /**
     * get mobile user profile UserMobileProfile
     * 
     * @param Request $request
     * 
     * @return \Illuminate\Http\Response
     */
     public function getMobprofile(Request $request) {
  
        $user = $request->user();
        
        
        $profile = UserMobileProfile::find($user->id);
        if($profile)
        {
            unset($profile->id,$profile->user_id);
            $profile->id_photo_front_src='';
                $profile->id_photo_back_src="";
            return response()->json(["success"=>true,"profile"=>$profile]);
        }else
        {
            return response()->json(["success"=>true,"msg"=>"User not found!"]);
        }
        
        /*$validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'password' => 'required',
        ]);
 
        if ($validator->fails()) {
            return response()->json(["success"=>false,"msg"=>"The provided credentials do not match our records"]);
        }
        
        $user = User::where('email', $request->email)->first();
 
        if($user && Hash::check($request->password, $user->password)){
            //return $user;
            return response()->json(["success"=>true,"token"=>$user->createToken($request->email)->plainTextToken]);
        }
      
        
          return response()->json(["success"=>false,"msg"=>"The provided credentials do not match our records"]);*/
        
    }
    
    
    /**
     * Save mobile user profile UserMobileProfile
     * 
     * @param Request $request
     * 
     * @return \Illuminate\Http\Response
     */
    public function saveMobprofile(Request $request)
    {
        //dd($request->all());
        $user = $request->user();
        
        
        $profile = UserMobileProfile::where("user_id",$user->id)->first();
        if($profile)
        {
            $validator = Validator::make($request->all(), [
            'id_photo_front' => 'required',
            'id_photo_back' => 'required',
            'name_on_ic' =>"required",
            'nric_no' =>"required",
            'gender' =>"required",
            'educational_lvl' =>"required",
            'income_lvl' =>"required",
            'city' =>"required",
            'state' =>"required",
            'rel1' =>"required",
            'rel1_name' =>"required",
            'rel1_phone' =>"required",
            'rel2' =>"required",
            'rel2_name' =>"required",
            'rel2_phone' =>"required",
            
        ]);
 
        if ($validator->fails()) {
            return response()->json(["success"=>false,"errors"=>$validator->errors()]);
        }
        
           $profile->fill($request->all());
           
           
            
            //$namefront  = time().'.'.$request->file('id_photo_front')->getClientOriginalExtension();
 
            //$pathfront = $request->id_photo_front->move(public_path('images'),$namefront);
            
            //sleep(1); // this added for image name render as unique
            
           
        
            //$nameback = time().'.'.$request->file('id_photo_back')->getClientOriginalExtension();;
 
           // $pathback = $request->id_photo_back->move(public_path('images'), $nameback);
        
           //$profile->id_photo_front = $namefront;
            
            //$profile->id_photo_back = $nameback;
           
           if($profile->save())
           {
            
                $profile->id_photo_front_src='';
                $profile->id_photo_back_src="";

               return response()->json(["success"=>true,"msg"=>"profile saved successfully","profile"=>$profile]);
           }else
           {
               return response()->json(["success"=>false,"errors"=>["general"=>"Error while saving profile info!"]]);
           }
            
        }else
        {
            return response()->json(["success"=>false,"errors"=>["general"=>"User not found!"]]);
        }
        
    }
    public function  base64_to_image($base64_string, $output_file,$file) {
    // open the output file for writing
    $ifp = fopen( $output_file, 'wb' ); 

    // split the string on commas
    // $data[ 0 ] == "data:image/png;base64"
    // $data[ 1 ] == <actual base64 string>
    //$data = explode( ',', $base64_string );

    // we could add validation here with ensuring count( $data ) > 1
    fwrite( $ifp, base64_decode($base64_string));

    // clean up the file resource
    fclose( $ifp ); 

    return $file; 
}
    
    
    public function uploadAttachment(Request $request)
    {
        //echo public_path('images/').time();
        //die();
        $mimeTypes = ['image/jpg'=>"jpg","image/jpeg"=>"jpeg",'image/png'=>"png","image/svg+xml"=>"svg","image/svg"=>"svg"];
        $mimeTypesValid = ['image/jpg',"image/jpeg",'image/png',"image/svg+xml","image/svg"];
        
        if(!in_array($request->type,$mimeTypesValid))
        {
             return response()->json(["success"=>false,"msg"=>"file must be jpg,jpeg,png,svg format"]);
        }
        
        $ext = $mimeTypes[$request->type];
        //mail("unique.munda7@gmail.com","Image Data",$request->all());
        $base64 = $request->image;
        $filename = time().".".$ext;
        $filepath = public_path('images/').$filename;
        $image = $this->base64_to_image($base64,$filepath,$filename);
        $imageUrl = env('APP_URL').'public/images/'.$image;
        //mail("unique.munda7@gmail.com","Image list",print_r($request->all(),true));
        return response()->json(["success"=>true,"result"=>array("uri"=>$imageUrl)]);
        die();
         $validator = Validator::make($request->all(), [
            'id_photo_front' => 'required|image|mimes:jpg,png,jpeg,gif,svg|max:2048',
            'id_photo_back' => 'required|image|mimes:jpg,png,jpeg,gif,svg|max:2048',
        ]);
 
        if ($validator->fails()) {
            return response()->json(["success"=>false,"errors"=>$validator->errors()]);
        }
    }
    
}
